package defi

import (
	"bytes"
	"encoding/json"
	"net/http"
	"net/url"
	"os"
	"strconv"
	"strings"

	"birdcall/internal/consts"
	"birdcall/internal/library/birdeye"
	"birdcall/internal/model/birdeye/defi"
)

type Defi struct{}

// 获取当前支持的网络
// @return *birdeye.CommonRes
// @return error
func (d *Defi) Support() (*defi.NetWorkRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(consts.DEFI_URL)
	fullURL := baseURL + consts.NET_WORKS

	apiReq, err := http.NewRequest(consts.GET, fullURL, nil)
	if err != nil {
		return nil, err
	}
	//构建并发送请求
	apiReq = birdeye.MakeHeader(apiReq)
	body, err := birdeye.SendRequest(apiReq)
	if err != nil {
		return nil, err
	}

	//处理返回数据
	var res *defi.NetWorkRes
	err = json.Unmarshal([]byte(body), &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}

//	 Price 获取token的价格
//		@param check_liquidity
//		@param include_liquidity
//		@param address
//		@return *birdeye.CommonRes
//		@return error
func (d *Defi) Price(chain_name string, check_liquidity float64, include_liquidity bool, address string) (*defi.PriceRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(consts.DEFI_URL)
	params := url.Values{}
	params.Add(consts.CheckLiquidity, strconv.FormatFloat(check_liquidity, 'f', 2, 64))
	params.Add(consts.IncludeLiquidity, strconv.FormatBool(include_liquidity))
	params.Add(consts.ADDRESS, address)
	fullURL := baseURL + consts.PRICE + consts.QUESTION_MARK + params.Encode()

	apiReq, err := http.NewRequest(consts.GET, fullURL, nil)
	if err != nil {
		return nil, err
	}

	//构建请求头
	apiReq = birdeye.MakeHeader(apiReq, chain_name)

	body, err := birdeye.SendRequest(apiReq)
	if err != nil {
		return nil, err
	}
	//处理返回数据
	var res *defi.PriceRes
	err = json.Unmarshal([]byte(body), &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}

// dc：一次获取多个token的价格
// @return *birdeye.CommonRes
// @return error
func (d *Defi) MultiplePrice(chain_name string, check_liquidity float64, include_liquidity bool, address []string, method string) (*defi.MultiplePriceRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(consts.DEFI_URL)
	params := url.Values{}
	params.Add(consts.CheckLiquidity, strconv.FormatFloat(check_liquidity, 'f', 2, 64))
	params.Add(consts.IncludeLiquidity, strconv.FormatBool(include_liquidity))

	var ApiReq *http.Request
	var payload *bytes.Buffer = nil

	//判断请求类型
	if method == consts.GET {
		addresses := strings.Join(address, consts.COMMAS)
		params.Add(consts.LIST_ADDRESS, addresses)
	} else if method == consts.POST {
		data, err := json.Marshal(address)
		if err != nil {
			return nil, err
		}
		payload = bytes.NewBuffer(data)
	}

	fullURL := baseURL + consts.MULTIPLE_PRICE + consts.QUESTION_MARK + params.Encode()

	if method == consts.GET {
		apiReq, err := http.NewRequest(method, fullURL, nil)
		if err != nil {
			return nil, err
		}
		ApiReq = apiReq
	} else if method == consts.POST {
		apiReq, err := http.NewRequest(method, fullURL, payload)
		if err != nil {
			return nil, err
		}
		ApiReq = apiReq
	}

	//构建请求头
	ApiReq = birdeye.MakeHeader(ApiReq, chain_name)

	//发送请求
	body, err := birdeye.SendRequest(ApiReq)
	if err != nil {
		return nil, err
	}
	var res *defi.MultiplePriceRes
	err = json.Unmarshal([]byte(body), &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}
